#include "TestFunctions.h"

uint16_t buffer[8000];

void DmaHalfTransferCompleteIsr() {
   Serial.println("DMA half transfer complete");
}

void DmaTransferCompleteIsr() {
   Serial.println("DMA transfer complete");
}

void AdcIsr(int count) {
   Serial.println(count);
}

void clear() {
   memset(buffer, 0xFF, sizeof(buffer));
}

void reportSingleChannel(const char* name, uint32_t time, int count) {
   Serial.printf("Test: %s\n", name);
   auto t = time / 8;
   Serial.printf("t: %s ns pro sample\n", UrsUtil.thousandSeparatorString(t).c_str());
   uint32_t f = (uint32_t)(1'000'000 / t);
   Serial.printf("f: %s ksps\n", UrsUtil.thousandSeparatorString(f).c_str());

   for (int i = 0; i < count; i++) {
      Serial.println(buffer[i] * 3.3 / 4096, 3);
   }
   Serial.println();
   delay(1000); // Ansonsten Pufferberlauf beim Consolen-Programm
}

void reportSequence(const char* name, uint32_t time, int count) {
   Serial.printf("Test: %s\n", name);
   if (time) {
      auto t = time / 8;
      Serial.printf("t: %s ns pro sample\n", UrsUtil.thousandSeparatorString(t).c_str());
      uint32_t f = (uint32_t)(1'000'000 / t);
      Serial.printf("f: %s ksps\n", UrsUtil.thousandSeparatorString(f).c_str());
   }

   Serial.println("   c0     c1     c2     c3     c4     c5     c6     c7");

   for (int i = 0; i < count; i++) {
      for (int j = 0; j < 8; j++) {
         Serial.print(buffer[i * 8 + j] * 3.3 / 4096, 3);
         Serial.print("  ");
      }
      Serial.println();
   }
   Serial.println();
   delay(2000); // Ansonsten Pufferberlauf beim Consolen-Programm
}

void reportSequence8Bit(const char* name, uint32_t time, int count) {
   Serial.printf("Test: %s\n", name);
   auto t = time / 8;
   Serial.printf("t: %s ns pro sample\n", UrsUtil.thousandSeparatorString(t).c_str());
   uint32_t f = (uint32_t)(1'000'000 / t);
   Serial.printf("f: %s ksps\n", UrsUtil.thousandSeparatorString(f).c_str());

   Serial.println("   c0     c1     c2     c3     c4     c5     c6     c7");

   uint8_t* bytebuffer = (uint8_t*)buffer;

   for (int i = 0; i < count; i++) {
      for (int j = 0; j < 8; j++) {
         Serial.print(bytebuffer[i * 8 + j] * 3.3 / 256, 3);
         Serial.print("  ");
      }
      Serial.println();
   }
   Serial.println();
   delay(3000); // Ansonsten Pufferberlauf beim Consolen-Programm
}


void testArduinoAnalogRead() {
   int m = 0;
   for (int k = 0; k < 1000; k++)
      for (int i = 0; i < 8; i++)
         buffer[m++] = analogRead(i);
}

void testUrsAdcAnalogRead() {
   int m = 0;
   for (int k = 0; k < 1000; k++)
      for (int i = 0; i < 8; i++)
         buffer[m++] = UrsADC.analogRead((UrsAdcChannel)i);
}

void testUrsAdcRead() {
   int m = 0;
   for (int k = 0; k < 1000; k++)
      for (int i = 0; i < 8; i++)
         buffer[m++] = UrsADC.repeatRead();
}


void testUrsAdcDmaRead() {
   UrsADC.startSequence({
      UrsAdcChannel::Channel0, UrsAdcChannel::Channel1, UrsAdcChannel::Channel2,
      UrsAdcChannel::Channel3, UrsAdcChannel::Channel4, UrsAdcChannel::Channel5,
      UrsAdcChannel::Channel6, UrsAdcChannel::Channel7 }, buffer, 1000
      );
   while (!UrsADC.isConversionComplete());
}

void testUrsAdcDmaReadAdcIrq() {
   UrsADC.startSequence({
      UrsAdcChannel::Channel0, UrsAdcChannel::Channel1, UrsAdcChannel::Channel2,
      UrsAdcChannel::Channel3, UrsAdcChannel::Channel4, UrsAdcChannel::Channel5,
      UrsAdcChannel::Channel6, UrsAdcChannel::Channel7 }, buffer, 10
#ifndef INSTALL_DMA_ISR
      , DmaTransferCompleteIsr, DmaHalfTransferCompleteIsr
#endif // NO_DMA_ISR
#ifndef INSTALL_ADC_ISR
      , AdcIsr
#endif //NO_ADC_ISR
      );
   while (!UrsADC.isConversionComplete());
}